package ru.ezsk.demo.graphLayout;

import static ru.ezsk.demo.graphLayout.XMath.minmax;
import static ru.ezsk.demo.graphLayout.XMath.sqr;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Shape;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.io.Serializable;
import java.util.Random;

/**
 * @version $Revision: 53 $ $Author: beholder $
 *          $Date: 2005-10-28 14:55:21 +0400 (Пт, 28 окт 2005) $
 */
public class Node implements Serializable
{
    //=========================================================== Class Fields

    private static final int SPEED_LIMIT = 5;

    private static final double SPEED_THRESHOLD = 1;

    private static final Color FIXED_COLOR = new Color(100, 220, 250);

    private static final Color SELECT_COLOR = Color.PINK;

    private static final Color NORMAL_COLOR = new Color(250, 220, 100);

    private static final Color BORDER_COLOR = Color.BLACK;

    private static final Color TEXT_COLOR = Color.BLACK;

    private static final int ARC_WIDTH  = 8;

    private static final int ARC_HEIGHT = 8;

    private static final Insets TEXT_INSETS = new Insets(2, 4, 2, 4);

    private static final int CIRCLE_DIAMETER = 12;

    //================================================================= Fields

    double x;

    double y;

    double dx;

    double dy;

    private boolean fixed;

    private String label;

    private transient RoundRectangle2D.Double roundRect;

    private transient GlyphVector glyphVector;

    private transient double labelWidth;

    private transient double labelHeight;

    private transient double glyphX;

    private transient double glyphY;

    private transient Shape shape;

    //=========================================================== Constructors

    public Node(String label) {
        this.label = label;

        roundRect = new RoundRectangle2D.Double();
        roundRect.arcwidth  = ARC_WIDTH;
        roundRect.archeight = ARC_HEIGHT;

        shape = roundRect;
    }

    public Node(double x, double y, String label) {
        this(label);
        this.x = x;
        this.y = y;
    }

    //================================================================ Methods

    public void move(Dimension bounds) {
        if (!fixed) {
            x += minmax(dx, -SPEED_LIMIT, SPEED_LIMIT);
            y += minmax(dy, -SPEED_LIMIT, SPEED_LIMIT);
        }
        x = minmax(x, 0, bounds.width);
        y = minmax(y, 0, bounds.height);
        dx /= 2;
        dy /= 2;

        //limitSpeed();
    }

    @SuppressWarnings({"UNUSED_SYMBOL"})
    private void limitSpeed() {
        if (dx < SPEED_THRESHOLD) {
            dx = 0.0;
        }
        if (dy < SPEED_THRESHOLD) {
            dy = 0.0;
        }
    }

    //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    public void paint(Graphics2D g, boolean picked, boolean hideName) {
        double gx = 0;
        double gy = 0;

        if (hideName) {
            shape = new Ellipse2D.Double(x - CIRCLE_DIAMETER / 2.0,
                                         y - CIRCLE_DIAMETER / 2.0,
                                         CIRCLE_DIAMETER,
                                         CIRCLE_DIAMETER);
        } else {
            if (glyphVector == null) {
                createGlyph(g);
            }
            gx = x - labelWidth  / 2.0;
            gy = y - labelHeight / 2.0;

            roundRect.x = gx;
            roundRect.y = gy;
            roundRect.width  = labelWidth  - 1;
            roundRect.height = labelHeight - 1;
            shape = roundRect;
        }

        g.setColor(picked ? SELECT_COLOR : (fixed ? FIXED_COLOR : NORMAL_COLOR));
        g.fill(shape);

        g.setColor(BORDER_COLOR);
        g.draw(shape);

        if (!hideName) {
            g.setColor(TEXT_COLOR);
            g.drawGlyphVector(glyphVector,
                              (float) (gx + TEXT_INSETS.left - glyphX),
                              (float) (gy + TEXT_INSETS.top  - glyphY));
        }
    }

    private void createGlyph(Graphics2D g) {
        final FontRenderContext frc = g.getFontRenderContext();
        glyphVector = g.getFont().createGlyphVector(frc, label);
        final Rectangle2D bounds = glyphVector.getLogicalBounds();

        labelWidth  = bounds.getWidth() +
                      TEXT_INSETS.left + TEXT_INSETS.right;
        labelHeight = bounds.getHeight() +
                      TEXT_INSETS.top + TEXT_INSETS.bottom;
        glyphX = bounds.getX();
        glyphY = bounds.getY();
    }

    //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    public void freeze() {
        dx = 0.0;
        dy = 0.0;
    }

    public void center(Dimension d) {
        x = d.width  / 2.0;
        y = d.height / 2.0;
    }

    public void shake(Random rng, int offset) {
        if (!fixed) {
            x += offset * 2 * rng.nextDouble() - offset;
            y += offset * 2 * rng.nextDouble() - offset;
        }
    }

    public void scramble(Random random, Dimension size, int xPad, int yPad) {
        if (!fixed) {
            x = xPad + (size.width  - xPad * 2) * random.nextDouble();
            y = yPad + (size.height - yPad * 2) * random.nextDouble();
        }
    }

    public double distSqrTo(int x, int y) {
        return sqr(this.x - x) + sqr(this.y - y);
    }

    public void toggleFixed() {
        fixed = !fixed;
    }

    public boolean contains(int x, int y) {
        return shape.contains(x, y);
    }

    public void setPoint(int x, int y) {
        this.x = x;
        this.y = y;
    }

    public void move(double dx, double dy) {
        x += dx;
        y += dy;
    }

    public void accelerate(double ax, double ay) {
        dx += ax;
        dy += ay;
    }

    //============================================================= Properties

    public boolean isFixed() {
        return fixed;
    }

    public void setFixed(boolean fixed) {
        this.fixed = fixed;
    }

    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
        glyphVector = null;
    }
}
